@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
+    Copyright (C) 1999, 2000, 2001, 2007 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,14 +14,23 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
-#include "config.h"
+/** \file filesys.c */
+
+/**
+ * \addtogroup PedFileSystem
+ *
+ * \note File systems exist on a PedGeometry - NOT a PedPartition.
+ *
+ * @{
+ */
+
+#include <config.h>
 
 #include <parted/parted.h>
 #include <parted/debug.h>
-#include <string.h>
 
 #if ENABLE_NLS
 #  include <libintl.h>
@@ -52,20 +61,24 @@ ped_file_system_type_unregister (PedFileSystemType* fs_type)
 	PedFileSystemType*	walk;
 	PedFileSystemType*	last = NULL;
 
+	PED_ASSERT (fs_types != NULL, return);
 	PED_ASSERT (fs_type != NULL, return);
 
-	for (walk = fs_types; walk != fs_type; walk = walk->next) {
-		if (!walk) return;
-		last = walk;
-	}
+	for (walk = fs_types; walk && walk != fs_type;
+                last = walk, walk = walk->next);
 
-	if (last) {
+	PED_ASSERT (walk != NULL, return);
+	if (last)
 		((struct _PedFileSystemType*) last)->next = fs_type->next;
-	} else {
-		fs_types = fs_types->next;
-	}
+	else
+		fs_types = fs_type->next;	
 }
 
+/**
+ * Get a PedFileSystemType by its @p name.
+ *
+ * @return @c NULL if none found.
+ */
 PedFileSystemType*
 ped_file_system_type_get (const char* name)
 {
@@ -80,6 +93,11 @@ ped_file_system_type_get (const char* name)
 	return walk;
 }
 
+/**
+ * Get the next PedFileSystemType after @p fs_type.
+ *
+ * @return @c NULL if @p fs_type is the last item in the list.
+ */
 PedFileSystemType*
 ped_file_system_type_get_next (const PedFileSystemType* fs_type)
 {
@@ -89,6 +107,14 @@ ped_file_system_type_get_next (const PedFileSystemType* fs_type)
 		return fs_types;
 }
 
+/**
+ * Attempt to find a file system and return the region it occupies.
+ *
+ * @param fs_type The file system type to probe for.
+ * @param geom The region to be searched.
+ *
+ * @return @p NULL if @p fs_type file system wasn't detected
+ */
 PedGeometry*
 ped_file_system_probe_specific (
 		const PedFileSystemType* fs_type, PedGeometry* geom)
@@ -130,8 +156,12 @@ _probe_with_open (PedGeometry* geom, int detected_count,
 
 	ped_device_open (geom->dev);
 
+	/* If one and only one file system that Parted is able to open
+	 * can be successfully opened on this geometry, return it.
+	 * If more than one can be, return NULL.
+	 */
 	for (i=0; i<detected_count; i++) {
-		if (detected [i]->ops->open && !_test_open (detected [i], geom))
+		if (!detected[i]->ops->open || !_test_open (detected [i], geom))
 			continue;
 
 		if (open_detected) {
@@ -142,6 +172,22 @@ _probe_with_open (PedGeometry* geom, int detected_count,
 		}
 	}
 
+	/* If no file system has been successfully opened, and
+	 * if Parted has detected at most one unopenable file system,
+	 * return it.
+	 */
+	if (!open_detected)
+	for (i=0; i<detected_count; i++) {
+		if (detected[i]->ops->open)
+			continue;
+		if (open_detected) {
+			ped_device_close (geom->dev);
+			return NULL;
+		} else {
+			open_detected = detected [i];
+		}
+	}	
+
 	ped_device_close (geom->dev);
 	return open_detected;
 }
@@ -185,6 +231,15 @@ _best_match (const PedGeometry* geom, PedFileSystemType* detected [],
 	return detected [best_match];
 }
 
+
+/**
+ * Attempt to detect a file system in region \p geom. 
+ * This function tries to be clever at dealing with ambiguous
+ * situations, such as when one file system was not completely erased before a
+ * new file system was created on top of it.
+ *
+ * \return a new PedFileSystem on success, \c NULL on failure
+ */
 PedFileSystemType*
 ped_file_system_probe (PedGeometry* geom)
 {
@@ -225,6 +280,15 @@ ped_file_system_probe (PedGeometry* geom)
 	return _probe_with_open (geom, detected_count, detected);
 }
 
+/**
+ * This function erases all file system signatures that indicate that a
+ * file system occupies a given region described by \p geom.
+ * After this operation ped_file_system_probe() won't detect any file system.
+ *
+ * \note ped_file_system_create() calls this before creating a new file system.
+ * 
+ * \return \c 1 on success, \c 0 on failure
+ */
 int
 ped_file_system_clobber (PedGeometry* geom)
 {
@@ -264,6 +328,10 @@ error:
 	return 0;
 }
 
+/* This function erases all signatures that indicate the presence of
+ * a file system in a particular region, without erasing any data
+ * contained inside the "exclude" region.
+ */
 static int
 ped_file_system_clobber_exclude (PedGeometry* geom,
 				 const PedGeometry* exclude)
@@ -283,6 +351,21 @@ ped_file_system_clobber_exclude (PedGeometry* geom,
 	return status;
 }
 
+/**
+ * This function opens the file system stored on \p geom, if it
+ * can find one.
+ * It is often called in the following manner:
+ * \code
+ * 	fs = ped_file_system_open (&part.geom)
+ * \endcode
+ *
+ * \throws PED_EXCEPTION_ERROR if file system could not be detected
+ * \throws PED_EXCEPTION_ERROR if the file system is bigger than its volume
+ * \throws PED_EXCEPTION_NO_FEATURE if opening of a file system stored on 
+ * 	\p geom is not implemented
+ *
+ * \return a PedFileSystem on success, \c NULL on failure.
+ */
 PedFileSystem*
 ped_file_system_open (PedGeometry* geom)
 {
@@ -309,7 +392,7 @@ ped_file_system_open (PedGeometry* geom)
 		if (ped_exception_throw (
 			PED_EXCEPTION_ERROR,
 			PED_EXCEPTION_IGNORE_CANCEL,
-			_("The file system is bigger than it's volume!"))
+			_("The file system is bigger than its volume!"))
 				!= PED_EXCEPTION_IGNORE)
 			goto error_destroy_probed_geom;
 	}
@@ -337,6 +420,16 @@ error:
 	return 0;
 }
 
+/**
+ * This function initializes a new file system of type \p type on 
+ * a region described by \p geom, writing out appropriate metadata and 
+ * signatures.  If \p timer is non-NULL, it is used as the progress meter.
+ *
+ * \throws PED_EXCEPTION_NO_FEATURE if creating file system type \p type 
+ * 	is not implemented yet
+ *
+ * \return a PedFileSystem on success, \c NULL on failure
+ */
 PedFileSystem*
 ped_file_system_create (PedGeometry* geom, const PedFileSystemType* type,
 			PedTimer* timer)
@@ -371,6 +464,11 @@ error:
 	return 0;
 }
 
+/**
+ * Close file system \p fs.
+ *
+ * \return \c 1 on success, \c 0 on failure
+ */
 int
 ped_file_system_close (PedFileSystem* fs)
 {
@@ -385,10 +483,17 @@ ped_file_system_close (PedFileSystem* fs)
 
 error_close_dev:
 	ped_device_close (dev);
-error:
 	return 0;
 }
 
+/**
+ * Check \p fs file system for errors.
+ *
+ * \throws PED_EXCEPTION_NO_FEATURE if checking file system \p fs is 
+ * 	not implemented yet
+ *
+ * \return \c 0 on failure (i.e. unfixed errors)
+ */
 int
 ped_file_system_check (PedFileSystem* fs, PedTimer* timer)
 {
@@ -489,6 +594,17 @@ error:
 	return NULL;
 }
 
+/**
+ * Create a new file system (of the same type) on \p geom, and
+ * copy the contents of \p fs into the new filesystem.  
+ * If \p timer is non-NULL, it is used as the progress meter.
+ *
+ * \throws PED_EXCEPTION_ERROR when trying to copy onto an overlapping partition
+ * \throws PED_EXCEPTION_NO_FEATURE if copying of file system \p fs 
+ * 	is not implemented yet
+ *
+ * \return a new PedFileSystem on success, \c NULL on failure
+ */
 PedFileSystem*
 ped_file_system_copy (PedFileSystem* fs, PedGeometry* geom, PedTimer* timer)
 {
@@ -527,7 +643,7 @@ ped_file_system_copy (PedFileSystem* fs, PedGeometry* geom, PedTimer* timer)
 				PED_EXCEPTION_CANCEL,
 				_("Direct support for copying file systems is "
 				  "not yet implemented for %s.  However, "
-				  "support for resizing implemented.  "
+				  "support for resizing is implemented.  "
 				  "Therefore, the file system can be copied if "
 				  "the new partition is at least as big as the "
 				  "old one.  So, either shrink the partition "
@@ -556,6 +672,18 @@ error:
 	return NULL;;
 }
 
+/**
+ * Resize \p fs to new geometry \p geom.
+ *
+ * \p geom should satisfy the ped_file_system_get_resize_constraint().
+ * (This isn't asserted, so it's not a bug not to... just it's likely
+ * to fail ;)  If \p timer is non-NULL, it is used as the progress meter.
+ *
+ * \throws PED_EXCEPTION_NO_FEATURE if resizing of file system \p fs 
+ * 	is not implemented yet
+ * 
+ * \return \c 0 on failure 
+ */
 int
 ped_file_system_resize (PedFileSystem* fs, PedGeometry* geom, PedTimer* timer)
 {
@@ -580,6 +708,14 @@ ped_file_system_resize (PedFileSystem* fs, PedGeometry* geom, PedTimer* timer)
 	return fs->type->ops->resize (fs, geom, timer);
 }
 
+/**
+ * This function returns a constraint on the region that all file systems
+ * of a particular type \p fs_type created on device \p dev with 
+ * ped_file_system_create() must satisfy. For example, FAT16 file systems must
+ * be at least 32 megabytes.
+ *
+ * \return \c NULL on failure
+ */
 PedConstraint*
 ped_file_system_get_create_constraint (const PedFileSystemType* fs_type,
 				       const PedDevice* dev)
@@ -591,7 +727,19 @@ ped_file_system_get_create_constraint (const PedFileSystemType* fs_type,
 		return NULL;
 	return fs_type->ops->get_create_constraint (dev);
 }
-
+/**
+ * Return a constraint, that represents all of the possible ways the
+ * file system \p fs can be resized with ped_file_system_resize().  
+ * This takes into account the amount of used space on
+ * the filesystem \p fs and the capabilities of the resize algorithm.
+ * Hints:
+ * -# if constraint->start_align->grain_size == 0, or
+ *    constraint->start_geom->length == 1, then the start can not be moved
+ * -# constraint->min_size is the minimum size you can resize the partition
+ *    to.  You might want to tell the user this ;-).
+ *    
+ * \return a PedConstraint on success, \c NULL on failure
+ */
 PedConstraint*
 ped_file_system_get_resize_constraint (const PedFileSystem* fs)
 {
@@ -602,6 +750,12 @@ ped_file_system_get_resize_constraint (const PedFileSystem* fs)
 	return fs->type->ops->get_resize_constraint (fs);
 }
 
+/**
+ * Get the constraint on copying \p fs with ped_file_system_copy()
+ * to somewhere on \p dev.
+ *
+ * \return a PedConstraint on success, \c NULL on failure
+ */ 
 PedConstraint*
 ped_file_system_get_copy_constraint (const PedFileSystem* fs,
 				     const PedDevice* dev)
@@ -625,3 +779,5 @@ ped_file_system_get_copy_constraint (const PedFileSystem* fs,
 
 	return NULL;
 }
+
+/** @} */
