@@ -4,7 +4,7 @@
     original version by Matt Domsch <Matt_Domsch@dell.com>
     Disclaimed into the Public Domain
 
-    Portions Copyright (C) 2001, 2002, 2003, 2005, 2006
+    Portions Copyright (C) 2001, 2002, 2003, 2005, 2006, 2007
         Free Software Foundation, Inc.
 
     EFI GUID Partition Table handling
@@ -26,7 +26,7 @@
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
-#include "config.h"
+#include <config.h>
 
 #include <parted/parted.h>
 #include <parted/debug.h>
@@ -34,8 +34,6 @@
 #include <parted/crc32.h>
 #include <inttypes.h>
 #include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
 #include <sys/types.h>
 #include <sys/ioctl.h>
 #include <fcntl.h>
@@ -62,7 +60,12 @@
 #define GPT_HEADER_REVISION_V0_99 0x00009900
 
 typedef uint16_t efi_char16_t;	/* UNICODE character */
-
+typedef struct _GuidPartitionTableHeader_t GuidPartitionTableHeader_t;
+typedef struct _GuidPartitionEntryAttributes_t GuidPartitionEntryAttributes_t;
+typedef struct _GuidPartitionEntry_t GuidPartitionEntry_t;
+typedef struct _PartitionRecord_t PartitionRecord_t;
+typedef struct _LegacyMBR_t LegacyMBR_t;
+typedef struct _GPTDiskData GPTDiskData;
 typedef struct {
         uint32_t time_low;
         uint16_t time_mid;
@@ -121,7 +124,7 @@ typedef struct {
 		    PED_CPU_TO_LE16 (0x11AA), 0xaa, 0x11, \
 		    { 0x00, 0x30, 0x65, 0x43, 0xEC, 0xAC }})
 
-typedef struct _GuidPartitionTableHeader_t {
+struct __attribute__ ((packed)) _GuidPartitionTableHeader_t {
 	uint64_t Signature;
 	uint32_t Revision;
 	uint32_t HeaderSize;
@@ -137,9 +140,9 @@ typedef struct _GuidPartitionTableHeader_t {
 	uint32_t SizeOfPartitionEntry;
 	uint32_t PartitionEntryArrayCRC32;
 	uint8_t* Reserved2;
-} __attribute__ ((packed)) GuidPartitionTableHeader_t;
+};
 
-typedef struct _GuidPartitionEntryAttributes_t {
+struct __attribute__ ((packed)) _GuidPartitionEntryAttributes_t {
 #ifdef __GNUC__ /* XXX narrow this down to !TinyCC */
 	uint64_t RequiredToFunction:1;
 	uint64_t Reserved:47;
@@ -151,16 +154,16 @@ typedef struct _GuidPartitionEntryAttributes_t {
 	uint32_t LOST:5;
         uint32_t GuidSpecific:16;
 #endif
-} __attribute__ ((packed)) GuidPartitionEntryAttributes_t;
+};
 
-typedef struct _GuidPartitionEntry_t {
+struct __attribute__ ((packed)) _GuidPartitionEntry_t {
 	efi_guid_t PartitionTypeGuid;
 	efi_guid_t UniquePartitionGuid;
 	uint64_t StartingLBA;
 	uint64_t EndingLBA;
 	GuidPartitionEntryAttributes_t Attributes;
 	efi_char16_t PartitionName[72 / sizeof(efi_char16_t)];
-} __attribute__ ((packed)) GuidPartitionEntry_t;
+};
 
 #define GPT_PMBR_LBA 0
 #define GPT_PMBR_SECTORS 1
@@ -181,40 +184,59 @@ typedef struct _GuidPartitionEntry_t {
          sizeof(GuidPartitionEntry_t))
 
 
-typedef struct _PartitionRecord_t {
-	uint8_t BootIndicator;	/* Not used by EFI firmware. Set to 0x80 to indicate that this
-				   is the bootable legacy partition. */
-	uint8_t StartHead;	/* Start of partition in CHS address, not used by EFI firmware. */
-	uint8_t StartSector;	/* Start of partition in CHS address, not used by EFI firmware. */
-	uint8_t StartTrack;	/* Start of partition in CHS address, not used by EFI firmware. */
-	uint8_t OSType;		/* OS type. A value of 0xEF defines an EFI system partition.
-				   Other values are reserved for legacy operating systems, and
-				   allocated independently of the EFI specification. */
-	uint8_t EndHead;	/* End of partition in CHS address, not used by EFI firmware. */
-	uint8_t EndSector;	/* End of partition in CHS address, not used by EFI firmware. */
-	uint8_t EndTrack;	/* End of partition in CHS address, not used by EFI firmware. */
-	uint32_t StartingLBA;	/* Starting LBA address of the partition on the disk. Used by
-				   EFI firmware to define the start of the partition. */
-	uint32_t SizeInLBA;	/* Size of partition in LBA. Used by EFI firmware to determine
-				   the size of the partition. */
-} __attribute__ ((packed)) PartitionRecord_t;
+struct __attribute__ ((packed)) _PartitionRecord_t {
+	/* Not used by EFI firmware. Set to 0x80 to indicate that this
+	   is the bootable legacy partition. */
+	uint8_t BootIndicator;
+
+	/* Start of partition in CHS address, not used by EFI firmware. */
+	uint8_t StartHead;
+
+	/* Start of partition in CHS address, not used by EFI firmware. */
+	uint8_t StartSector;
+
+	/* Start of partition in CHS address, not used by EFI firmware. */
+	uint8_t StartTrack;
+
+	/* OS type. A value of 0xEF defines an EFI system partition.
+	   Other values are reserved for legacy operating systems, and
+	   allocated independently of the EFI specification. */
+	uint8_t OSType;
+
+	/* End of partition in CHS address, not used by EFI firmware. */
+	uint8_t EndHead;
+
+	/* End of partition in CHS address, not used by EFI firmware. */
+	uint8_t EndSector;
+
+	/* End of partition in CHS address, not used by EFI firmware. */
+	uint8_t EndTrack;
+
+	/* Starting LBA address of the partition on the disk. Used by
+	   EFI firmware to define the start of the partition. */
+	uint32_t StartingLBA;
+
+	/* Size of partition in LBA. Used by EFI firmware to determine
+	   the size of the partition. */
+	uint32_t SizeInLBA;
+};
 
 /* Protected Master Boot Record  & Legacy MBR share same structure */
 /* Needs to be packed because the u16s force misalignment. */
-typedef struct _LegacyMBR_t {
+struct __attribute__ ((packed)) _LegacyMBR_t {
 	uint8_t BootCode[440];
 	uint32_t UniqueMBRSignature;
 	uint16_t Unknown;
 	PartitionRecord_t PartitionRecord[4];
 	uint16_t Signature;
-} __attribute__ ((packed)) LegacyMBR_t;
+};
 
 /* uses libparted's disk_specific field in PedDisk, to store our info */
-typedef struct _GPTDiskData {
+struct __attribute__ ((packed)) _GPTDiskData {
 	PedGeometry	data_area;
 	int		entry_count;
 	efi_guid_t	uuid;
-} GPTDiskData;
+};
 
 /* uses libparted's disk_specific field in PedPartition, to store our info */
 typedef struct _GPTPartitionData {
@@ -341,7 +363,6 @@ pth_get_raw (const PedDevice* dev, const GuidPartitionTableHeader_t* pth)
 static void
 swap_uuid_and_efi_guid(uuid_t uuid)
 {
-	int i;
 	efi_guid_t *guid = (efi_guid_t *)uuid;
 
 	PED_ASSERT(uuid != NULL, return);
@@ -456,7 +477,6 @@ gpt_clobber(PedDevice * dev)
         uint8_t* zeroed_pth_raw = ped_malloc (pth_get_size (dev));
         uint8_t* pth_raw = ped_malloc (pth_get_size (dev));
 	GuidPartitionTableHeader_t* gpt;
-	GuidPartitionEntry_t ptes[GPT_DEFAULT_PARTITION_ENTRIES];
 
 	PED_ASSERT (dev != NULL, return 0);
 
@@ -607,17 +627,20 @@ _read_header (const PedDevice* dev, GuidPartitionTableHeader_t** gpt,
 }
 
 static int
-_parse_header (PedDisk* disk, GuidPartitionTableHeader_t* gpt)
+_parse_header (PedDisk* disk, GuidPartitionTableHeader_t* gpt, 
+	       int *update_needed)
 { 
 	GPTDiskData* gpt_disk_data = disk->disk_specific;
 	PedSector first_usable;
 	PedSector last_usable;
+	PedSector last_usable_if_grown, last_usable_min_default;
+	static int asked_already;
 
 	PED_ASSERT (_header_is_valid (disk->dev, gpt), return 0);
 
 #ifndef DISCOVER_ONLY
-	if (PED_CPU_TO_LE32 (gpt->Revision) > GPT_HEADER_REVISION_V1_02
-	    || PED_CPU_TO_LE32 (gpt->HeaderSize) != pth_get_size_static (
+	if (PED_LE32_TO_CPU (gpt->Revision) > GPT_HEADER_REVISION_V1_02
+	    || PED_LE32_TO_CPU (gpt->HeaderSize) != pth_get_size_static (
                                                         disk->dev)) {
 		if (ped_exception_throw (
 			PED_EXCEPTION_WARNING,
@@ -625,21 +648,78 @@ _parse_header (PedDisk* disk, GuidPartitionTableHeader_t* gpt)
 			_("The format of the GPT partition table is version "
 			  "%x, which is newer than what Parted can "
 			  "recognise.  Please tell us!  bug-parted@gnu.org"),
-			PED_CPU_TO_LE32 (gpt->Revision))
+			PED_LE32_TO_CPU (gpt->Revision))
 				!= PED_EXCEPTION_IGNORE)
 			return 0;
 	}
 #endif
 
-	first_usable = PED_CPU_TO_LE64 (gpt->FirstUsableLBA);
-	last_usable = PED_CPU_TO_LE64 (gpt->LastUsableLBA);
+	first_usable = PED_LE64_TO_CPU (gpt->FirstUsableLBA);
+	last_usable = PED_LE64_TO_CPU (gpt->LastUsableLBA);
+
+
+/*
+   Need to check whether the volume has grown, the LastUsableLBA is
+   normally set to disk->dev->length - 2 - ptes_size (at least for parted
+   created volumes), where ptes_size is the number of entries * 
+   size of each entry / sector size or 16k / sector size, whatever the greater.
+   If the volume has grown, offer the user the chance to use the new
+   space or continue with the current usable area.  Only ask once per 
+   parted invocation.
+*/
+   
+	last_usable_if_grown 
+		= PED_CPU_TO_LE64 (disk->dev->length - 2 - 
+		((PedSector)(PED_LE32_TO_CPU(gpt->NumberOfPartitionEntries)) * 
+		(PedSector)(PED_LE32_TO_CPU(gpt->SizeOfPartitionEntry)) / 
+		disk->dev->sector_size));
+
+	last_usable_min_default = disk->dev->length - 2 - 
+		GPT_DEFAULT_PARTITION_ENTRY_ARRAY_SIZE / disk->dev->sector_size;
+
+	if ( last_usable_if_grown > last_usable_min_default ) {
+
+		last_usable_if_grown = last_usable_min_default;
+	}
+
+
 	PED_ASSERT (last_usable > first_usable, return 0);
 	PED_ASSERT (last_usable <= disk->dev->length, return 0);
+
+	PED_ASSERT (last_usable_if_grown > first_usable, return 0);
+	PED_ASSERT (last_usable_if_grown <= disk->dev->length, return 0);
+
+	if ( !asked_already && last_usable < last_usable_if_grown ) {
+
+		PedExceptionOption q;
+
+		q = ped_exception_throw (PED_EXCEPTION_WARNING,
+			PED_EXCEPTION_FIX | PED_EXCEPTION_IGNORE,
+			_("Not all of the space available to %s appears "
+			"to be used, you can fix the GPT to use all of the "
+			"space (an extra %llu blocks) or continue with the "
+			"current setting? "), disk->dev->path, 
+			(uint64_t)(last_usable_if_grown - last_usable));
+
+
+		if (q == PED_EXCEPTION_FIX) {
+
+			last_usable = last_usable_if_grown;
+			*update_needed = 1;
+
+		}
+		else if (q != PED_EXCEPTION_UNHANDLED ) {
+
+			asked_already = 1;
+		}
+	}
+
 	ped_geometry_init (&gpt_disk_data->data_area, disk->dev,
 			   first_usable, last_usable - first_usable + 1);
 
+
 	gpt_disk_data->entry_count
-		= PED_CPU_TO_LE32 (gpt->NumberOfPartitionEntries);
+		= PED_LE32_TO_CPU (gpt->NumberOfPartitionEntries);
 	PED_ASSERT (gpt_disk_data->entry_count > 0, return 0);
 	PED_ASSERT (gpt_disk_data->entry_count <= 8192, return 0);
 
@@ -795,7 +875,7 @@ gpt_read (PedDisk * disk)
 		}
 	}
 
-	if (!_parse_header (disk, gpt))
+	if (!_parse_header (disk, gpt, &write_back))
 		goto error_free_gpt;
 
 
@@ -870,7 +950,7 @@ _write_pmbr (PedDevice * dev)
 }
 
 static void
-_generate_header (PedDisk* disk, int alternate, uint32_t ptes_crc,
+_generate_header (const PedDisk* disk, int alternate, uint32_t ptes_crc,
 		  GuidPartitionTableHeader_t** gpt_p)
 {
 	GPTDiskData* gpt_disk_data = disk->disk_specific;
@@ -882,7 +962,9 @@ _generate_header (PedDisk* disk, int alternate, uint32_t ptes_crc,
         
 	gpt->Signature = PED_CPU_TO_LE64 (GPT_HEADER_SIGNATURE);
 	gpt->Revision = PED_CPU_TO_LE32 (GPT_HEADER_REVISION_V1_00);
-	gpt->HeaderSize = PED_CPU_TO_LE32 (pth_get_size_static (disk->dev)); /* per 1.00 spec */
+
+	/* per 1.00 spec */
+	gpt->HeaderSize = PED_CPU_TO_LE32 (pth_get_size_static (disk->dev));
 	gpt->HeaderCRC32 = 0;
 	gpt->Reserved1 = 0;
 
@@ -893,7 +975,7 @@ _generate_header (PedDisk* disk, int alternate, uint32_t ptes_crc,
 		gpt->MyLBA = PED_CPU_TO_LE64 (disk->dev->length - 1);
 		gpt->AlternateLBA = PED_CPU_TO_LE64 (1);
 		gpt->PartitionEntryLBA
-			= PED_CPU_TO_LE64 (disk->dev->length - 2 - ptes_size);
+			= PED_CPU_TO_LE64 (disk->dev->length - 1 - ptes_size);
 	} else {
 		gpt->MyLBA = PED_CPU_TO_LE64 (1);
 		gpt->AlternateLBA = PED_CPU_TO_LE64 (disk->dev->length - 1);
@@ -935,7 +1017,7 @@ _partition_generate_part_entry (PedPartition* part, GuidPartitionEntry_t* pte)
 }
 
 static int
-gpt_write(PedDisk * disk)
+gpt_write(const PedDisk * disk)
 {
 	GPTDiskData* gpt_disk_data;
 	GuidPartitionEntry_t* ptes;
@@ -944,7 +1026,6 @@ gpt_write(PedDisk * disk)
 	GuidPartitionTableHeader_t* gpt;
 	PedPartition* part;
 	int ptes_size;
-	unsigned int i;
 
 	PED_ASSERT (disk != NULL, goto error);
 	PED_ASSERT (disk->dev != NULL, goto error);
@@ -984,7 +1065,7 @@ gpt_write(PedDisk * disk)
 	if (!ped_device_write (disk->dev, pth_raw, disk->dev->length - 1, 1))
 		goto error_free_ptes;
 	if (!ped_device_write (disk->dev, ptes,
-			       disk->dev->length - 2 - ptes_size / disk->dev->sector_size,
+			       disk->dev->length - 1 - ptes_size / disk->dev->sector_size,
 			       ptes_size / disk->dev->sector_size))
 		goto error_free_ptes;
 
@@ -1018,7 +1099,6 @@ add_metadata_part(PedDisk * disk, PedSector start, PedSector length)
 
 error_destroy_constraint:
 	ped_constraint_destroy (constraint_exact);
-error_destroy_part:
 	ped_partition_destroy (part);
 error:
 	return 0;
@@ -1167,7 +1247,8 @@ gpt_alloc_metadata (PedDisk * disk)
 	PED_ASSERT(disk->disk_specific != NULL, return 0);
 	gpt_disk_data = disk->disk_specific;
 
-	gptlength = ped_div_round_up (sizeof (GuidPartitionTableHeader_t), disk->dev->sector_size);
+	gptlength = ped_div_round_up (sizeof (GuidPartitionTableHeader_t),
+	                              disk->dev->sector_size);
 	pteslength = ped_div_round_up (gpt_disk_data->entry_count
 				       * sizeof (GuidPartitionEntry_t), disk->dev->sector_size);
 
@@ -1202,7 +1283,9 @@ gpt_partition_enumerate (PedPartition* part)
 		}
 	}
 
-	PED_ASSERT (0, return 0); return 0;
+	PED_ASSERT (0, return 0);
+
+	return 0; /* used if debug is disabled */
 }
 
 static int
@@ -1408,15 +1491,14 @@ static PedDiskType gpt_disk_type = {
 void
 ped_disk_gpt_init()
 {
-	PED_ASSERT(sizeof(GuidPartitionEntryAttributes_t) == 8, return);
-	PED_ASSERT(sizeof(GuidPartitionEntry_t) == 128, return);
+	PED_ASSERT (sizeof (GuidPartitionEntryAttributes_t) == 8, return);
+	PED_ASSERT (sizeof (GuidPartitionEntry_t) == 128, return);
 
-	ped_register_disk_type(&gpt_disk_type);
+	ped_disk_type_register (&gpt_disk_type);
 }
 
 void
 ped_disk_gpt_done()
 {
-	ped_unregister_disk_type(&gpt_disk_type);
+	ped_disk_type_unregister (&gpt_disk_type);
 }
-
