@@ -1,6 +1,6 @@
 /*
     parted - a frontend to libparted
-    Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
+    Copyright (C) 1999, 2000, 2001, 2007 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
 #ifndef UI_H_INCLUDED
@@ -22,9 +22,6 @@
 
 #include "strlist.h"
 
-#define MEGABYTE (1024 * 1024)
-#define MEGABYTE_SECTORS (MEGABYTE / 512)
-
 extern char*	prog_name;
 
 extern int init_ui ();
@@ -47,26 +44,26 @@ extern void command_line_prompt_words (const char* prompt, const char* def,
 extern char* command_line_get_word (const char* prompt, const char* def,
 				    const StrList* possibilities,
 				    int multi_word);
-extern int command_line_get_integer (const char* prompt, int def, int* result);
-extern PedSector command_line_get_sector (const char* prompt, PedDevice* dev,
-					  PedSector def, PedSector* result);
-extern int command_line_get_state (const char* prompt, int def, int* result);
-extern PedDevice* command_line_get_device (const char* prompt, PedDevice* def);
-extern PedDisk* command_line_get_disk (const char* prompt, PedDisk* def);
-extern PedPartition* command_line_get_partition (const char* prompt,
-						 PedDisk* disk);
-extern const PedFileSystemType* command_line_get_fs_type (const char* prompt,
-					const PedFileSystemType* def);
-extern const PedDiskType* command_line_get_disk_type (const char* prompt,
-					const PedDiskType* def);
+extern int command_line_get_integer (const char* prompt, int* value);
+extern int command_line_get_sector (const char* prompt, PedDevice* dev,
+				    PedSector* value, PedGeometry** range);
+extern int command_line_get_state (const char* prompt, int* value);
+extern int command_line_get_device (const char* prompt, PedDevice** value);
+extern int command_line_get_disk (const char* prompt, PedDisk** value);
+extern int command_line_get_partition (const char* prompt, PedDisk* disk,
+				       PedPartition** value);
+extern int command_line_get_fs_type (const char* prompt,
+				     const PedFileSystemType*(* value));
+extern int command_line_get_disk_type (const char* prompt,
+				       const PedDiskType*(* value));
 extern int command_line_get_part_flag (const char* prompt,
 				       const PedPartition* part,
 				       PedPartitionFlag* flag);
-extern int command_line_get_new_part_type (const char* prompt,
-					   const PedDisk* disk,
-					   PedPartitionType* type);
+extern int command_line_get_part_type (const char* prompt, const PedDisk* disk,
+				       PedPartitionType* type);
 extern PedExceptionOption command_line_get_ex_opt (const char* prompt,
 						   PedExceptionOption options);
+extern int command_line_get_unit (const char* prompt, PedUnit* unit);
 
 extern int command_line_is_integer ();
 extern int command_line_is_sector ();
@@ -77,10 +74,10 @@ extern void print_using_dev (PedDevice* dev);
 
 /* in parted.c */
 extern int	opt_script_mode;
+extern int	pretend_input_tty;
 
 extern void print_options_help ();
 extern void print_commands_help ();
 
 
 #endif /* UI_H_INCLUDED */
-
